def exec_no_show(filepath, globals=None, locals=None):
    """
    exec a file except for plt.show
    """
    if globals is None:
        globals = {}
    globals.update(
        {
            "__file__": filepath,
            "__name__": "__main__",
        }
    )
    with open(filepath) as file:
        block = file.read()
        block = block.replace("plt.show", "")
        exec(compile(block, filepath, "exec"), globals)
    return globals



import json
import os
from unittest import mock

import matplotlib
import numpy as np
from matplotlib import animation
from .util import exec_no_show
from ._version import schema_version

_prog_bar = True
try:
    from tqdm import tqdm
except ImportError:
    _prog_bar = False


matplotlib.use("agg")

__all__ = [
    "gen_mock_events",
    "load_events",
    "playback_file",
    "playback_events",
]


def gen_mock_events(events, globals):
    mock_events = []
    for event in events:
        mock_event = mock.Mock()
        for k, v in event.items():
            if k == "fig":
                setattr(mock_event, "canvas", globals[v].canvas)
            elif k == "inaxes":
                setattr(mock_event, "inaxes", globals.get(v, None))
            else:
                setattr(mock_event, k, v)
        mock_events.append(mock_event)
    return mock_events


def load_events(events):
    meta = {}
    if isinstance(events, str):
        with open(events) as f:
            loaded = json.load(f)
            meta["figname"] = loaded["figname"]
            meta["schema-version"] = loaded["schema-version"]
            events = loaded["events"]
    return meta, events


def playback_file(events, path, output, prog_bar=True, **kwargs):
    """
    Parameters
    ----------
    events : str
        Path to the json file defining the events or a dictionary of an
        already loaded file.
    path : str
        path to the file to be executed.
    output : str
        The path to the output file
    prog_bar : bool, default: True
        Whether to display a progressbar of animation progress.
    **kwargs :
        Passed through to `FuncAnimation`.
    """
    meta, events = load_events(events)
    figname = meta["figname"]
    gbl = exec_no_show(path)
    playback_events(figname, events, gbl, output, prog_bar=prog_bar, **kwargs)


def playback_events(figname, events, globals, output, prog_bar=True, **kwargs):
    """
    plays back events that have been

    Parameters
    ----------

    """
    mock_events = gen_mock_events(events, globals)

    # need to use transforms better probs need to record the x/y in
    # figure coordinates, then convert back to display coords for mocking
    # the events

    # Here use the last axis in order to get a high zorder
    (fake_mouse,) = (
        globals[figname]
        .axes[-1]
        .plot(
            [0, 5], [0, 1], "k", marker=6, markersize=15, transform=None, clip_on=False
        )
    )

    if prog_bar and _prog_bar:
        pbar = tqdm(total=len(events))

    def init():
        pass

    def animate(i):
        event = mock_events[i]
        globals[figname].canvas.callbacks.process(event.name, event)
        if event.name == "motion_notify_event":
            fake_mouse.set_data(event.x, event.y)

        globals[figname].canvas.draw()
        if prog_bar and _prog_bar:
            pbar.update(1)

    interval = kwargs.pop("interval", 40)
    ani = animation.FuncAnimation(
        globals[figname],
        animate,
        range(len(events)),
        init_func=init,
        interval=interval,
        **kwargs
    )
    dirname = os.path.dirname(output)
    if not os.path.exists(dirname) and dirname not in ["", "."]:
        os.makedirs(os.path.dirname(output))
    ani.save(output)



import collections
import json
from functools import partial
from os import path
from pathlib import Path

from .util import exec_no_show
from ._version import schema_version

__all__ = [
    "possible_events",
    "record_events",
    "record_file",
    "record_figure",
]
possible_events = [
    "button_press_event",
    "button_release_event",
    "draw_event",
    "key_press_event",
    "key_release_event",
    "motion_notify_event",
    "pick_event",
    "resize_event",
    "scroll_event",
    "figure_enter_event",
    "figure_leave_event",
    "axes_enter_event",
    "axes_leave_event",
]

event_list = []


def print_event(event, fig, inv_locals):
    info2save = [
        e for e in dir(event) if "_" not in e and e not in ["guiEvent", "lastevent"]
    ]
    saved_info = {k: getattr(event, k) for k in info2save}
    saved_info.pop("canvas")
    saved_info["fig"] = inv_locals[fig]
    saved_info["inaxes"] = inv_locals.get(saved_info["inaxes"], None)
    event_list.append(saved_info)


def record_file(path, figname="fig"):
    """
    Parameters
    ----------

    path : str
        The path to the file to be run
    figname : str, default: fig
        The variable name of the figure to capture
    output : str or None, default: None
        Defaults to ``_<file-name>-playback.json``
    """
    globals = exec_no_show(path)

    out = "_" + Path(path).stem + "-playback.json"
    record_figure(figname, globals, out)


def record_figure(figname, globals, savename):
    """
    Parameters
    ----------
    fig : str
        The variable name of the figure
    globals : dict
    savename : str
    """
    inv_globals = {
        v: k for k, v in globals.items() if isinstance(v, collections.abc.Hashable)
    }
    record_events(
        globals[figname],
        ["motion_notify_event", "button_press_event", "button_release_event"],
        inv_globals,
    )
    globals["plt"].show()
    with open(savename, "w") as fp:
        json.dump(
            {
                "figname": figname,
                "schema-version": schema_version,
                "events": event_list,
            },
            fp,
        )


def record_events(fig, events, inv_locals):
    if isinstance(events, str):
        events = [events]
    for e in events:
        fig.canvas.mpl_connect(e, partial(print_event, fig=fig, inv_locals=inv_locals))



from os import path

from setuptools import find_packages, setup

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# extract version
path = path.realpath("mpl_playback/_version.py")
version_ns = {}
with open(path, encoding="utf8") as f:
    exec(f.read(), {}, version_ns)
version = version_ns["__version__"]

name = "mpl-playback"

setup_args = dict(
    name=name,
    version=version,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=[
        "matplotlib>=3.3",
    ],
    author="Ian Hunt-Isaak",
    author_email="ianhuntisaak@gmail.com",
    license="BSD 3-Clause",
    platforms="Linux, Mac OS X, Windows",
    keywords=["Matplotlib"],
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Framework :: Jupyter",
        "Framework :: Matplotlib",
    ],
    url="https://github.com/ianhi/mpl-interactions",
    extras_require={
        "doc": [
            "sphinx>=1.5",
            "sphinx-copybutton",
            "sphinx_gallery",
        ],
        "test": [
            "black",
        ],
    },
)

if __name__ == "__main__":
    setup(**setup_args)
